## Books Contain Multitudes: A Scholarly Compendium for Experimental Book Publishing. 

Scholars, publishers, infrastructure providers, developers, and librarians, are increasingly experimenting with alternative tools, practices, and formats for scholarly publishing. This field of work has been growing with the emergence of digital and open access publishing, yet those who want to experiment with the forms, practices, and systems of academic publishing still struggle to find appropriate publishers, tools, examples, and communities that can support their ventures. Others might simply become disillusioned, put off perhaps by the apparent disconnect between the desire to experiment with forms, media and formats of scholarly books and the unfamiliarity amongst scholars and publishers with the tools, technologies, and publishing and funding opportunities needed to do so. Furthermore, monograph publishing in the humanities has been embedded in an individual and competitive academic culture that, as Samuel Moore (2019) underlines, has a strong ordering effect on both academics and the way in which they publish: “projects are undertaken with specific publication formats in mind; journal choice is frequently determined by how well regarded they are by assessment panels; and there is an informal hierarchy of certain kinds of academic publication, from the monograph at the top down to co-authored works and book chapters in edited volumes towards the bottom” (41).

Having worked on experimental book publishing as part of a community of researchers, authors, developers, editors, librarians and publishers collaborating in the framework of the COPIM project, we compiled this compendium to address this gap. The compendium is based on and expands from two reports we developed within COPIM's Experimental Publishing Group. We want the compendium to act as a guide and reference for both experienced practitioners and those just setting out to experiment with the form, content, and practices of scholarly bookmaking. The compendium includes experiments with the form and format of the scholarly book; with the various (multi)media we can publish long-form research in; and with how people produce, disseminate, consume, review, reuse, interact with and form communities around books. Far from being merely a formal exercise, **experimental publishing as we conceive it here also reimagines the relationalities that constitute scholarly writing, research, and publishing.** Books, after all, validate what counts as research and materialise how scholarly knowledge production is organised. The linked entries in this compendium may inspire speculations on the future of the book and the humanities more in general and encourage publishers and authors to explore publications beyond the standard printed codex format. 

This compendium has been compiled by Janneke Adema, Simon Bowie, Gary Hall, Rebekka Kiesewetter, Julien McHardy, and Tobias Steiner. Having experimented with books for many years in various constellations we came together in COPIM’s Experimental Publishing Group for a period of three-and-a-half-years. During this time we collaborated with authors, tool and platform providers, designers, publishers, librarians, and developers, working together to create a number of pilot projects and research reports. This compendium gathers insights from the process of producing these resources, reflecting on our practice and that of our collaborators. Given that experimental publishing is continually-emergent and diverse, and that our understanding will always be partial, contradictory, and situated we hope that this compendium will evolve with those who interact with it, with the communities we created around our publishing experiments, and the communities that are still to come. In this introduction, we are sharing how this compendium came about and how we hope to further develop it, amongst others by inviting amendments by all who carry it forward by interacting with it, and by ongoing maintenance and (re-)use. 

### Books are Political

Our starting premise is that scholarly bookmaking is political (Adema and Hall, 2015). Far from coming at the end of academic work, books contain multitudes, manifesting the entire field of scholarly knowledge production. How research is conceptualised, funded, conducted, analysed, articulated, valued and shared affects the shape and form of academic books. Experimenting with academic books, in turn, is to experiment with how academic labour and research are organised. From this starting point, we propose that experimenting with academic books is never just a formal exercise. Publishing is *one* site where scholars might intervene and reconfigure the knowledge production process. The idea that seemingly technical practices and infrastructures are political is a core insight of Science and Technology Studies (STS). Still, until recently, the critical study of knowledge infrastructures has long refrained from reflecting on our own research practices, to look inward to explore how academic publishing infrastructures shape academic realities. Many scholar-led publishing intitiatives take up this challenge. Involving scholars in the critical remaking of academic publishing emphasises that publishing is neither merely a service nor an end product, but an integral part of how research gets done. There cannot be *one* uniform 'best' way to organise academic publishing. Acknowledging - together with e.g. Haraway (1988) and Chan et al. (2019) - the contextual and situated nature of knowledge production, and moving away from notions of 'best' practice, we recognise that different research endeavours and academic communities might require a variety of forms of, and approaches to publishing, which is where COPIM — the Community-led Open Publication Infrastructures for Monographs project — comes in.

As commercial consolidation continues to threaten to monopolise and homogenise the emerging scholarly Open Access publishing landscape, COPIM's mission was to gather researchers, publishers, libraries and infrastructure providers to develop community-owned ***alternatives to established infrastructure*** that can support and scale a diverse publishing landscapes. Open *infrastructures*, we proposed, can provide an alternative to platform capitalism that extracts value by monopolising access. Under the banner of Scaling Small, COPIM worked towards a diverse publishing landscape prioritising community ownership, collective production and governance, scholar-led publishing, and sharing resources and open infrastructures amongst various institutions guided by the project's core value of Bibliophilia (love and care for books). COPIM’s work packages were largely dedicated to serious infrastructure building, except for the experimental publishing group, which grants the question of how experimental publishing contributes to the ambition to establish infrastructures that allow diverse *small* initiatives to proliferate at *scale*?

The image of diverse publishing landscapes or interrelated publishing ecosystems helped position COPIM’s work. These ecological metaphors evoke publishing as a relational enterprise, more or less explicitly valuing diversity over monoculture. Staying with metaphors of lively, specific and abundant interdependences allows us to sketch our understanding of experimental publishing’s place in scholarly knowledge production. Speaking of publishing ecologies reiterates our point that scholarly publishing cannot be separated from how it is funded, conceptualised, written, valued, reviewed, rewarded, read and taught. Scholarly publishing ecologies reflect and materialise the broader scholarly landscape. In this ecological view, scholarly books are not containers of knowledge but relational nodes that materialise what does and doesn’t count as valuable practices, sites, labour, and subjects of knowledge. In this metaphor, scholarly works, like all specimens, coevolve with the environment they inhabit and shape. In contrast to ecological metaphors, where knowledge circulates in diverse, partially connected patterns, schematic understandings of academic knowledge production frequently liken the flow of knowledge to managed water. For example, grant applications and project timetables tend to imagine scholarly publishing at the end of a pipeline. How institutions such as libraries, universities, publishers, funders, and intellectual property regimes organise knowledge production tends to reinforce the notion of a manageable flow from funding to research question to investigation to publication to evaluation to more funding. The metaphor of channelled flow and the premises of contained stages provides structure. The notion that valuable knowledge may be channeled, gives publishing a place and a form: the book coming at the end of the pipe. But, you see it coming, where there are pipes, there are blockages and spillage. Coming back to experimental publishing and ecological metaphors of flow, new forms of publication might streamline or spill over into relational circulations that have more in common with moors, swamps, and wetlands than pipes. Either way, we posit that experimental publishing is one of the sites where the shape of scholarly landscapes, and their relationship to other ecologies of knowledge and power is negotiated and materialised in practice. How we *do* publish matters. Experimenting with scholarly books is to experiment with scholarly modes of knowledge production. This labour, like other experimental practice, takes place at the growing edges and in the cracks of established practices, where by steady erosion, underground commotion or capital-intense incubation forms of writing, making, sharing, reviewing, discovering, reading and cataloging books come into being that will question, and occasionally change what counts as scholarly work. 

The compendium gathers and links tools, examples and practices with a focus on free and open-source software, platforms and digital publishing tools that presses and authors can either use freely and/or further adapt themselves to their workflows. The compendium also proposes a typology of experimental publishing. This typology is a starting point for exploration, rather than a fixed classification. It doesn’t provide clear definitions of the various experiments undertaken within scholarly book publishing—if only because many of the examples will defy categories. This mapping provides a snapshot, a temporary overview and analysis, one that will hopefully be updated, revised, and reused in different contexts. In this respect analysing experimental publishing—perhaps more than established forms of publishing—requires a continuous re-mapping due to the nature of its speculative and emergent form, where any map will need to be repeatedly redrawn if we want to analyse experimental publishing’s material-discursive practices. At the same time, we are aware of the performative character of our analyses (i.e., how any classification we suggest will provide further authority and weight to that classification), which will inherently be a factor in the stabilising, fixing, and freezing of these practices and knowledge relations, including as part of the mapping or typology that we provide here.

By keeping this mapping open, both for updates and further uptake by the community, we hope we can prevent a too stringently fixing-down of the speculative character of these experiments, where instead we want to emphasise that its political nature lies in the book continuing ‘to be able to serve ‘‘new ends’’ as a medium through which politics itself can be rethought’ (Adema and Hall 2013; Drucker, 2004). Indeed, experimental publishing can be seen as an attempt to keep ‘open the politics of knowledge and communication in a context where these are being closed down’. Following this line of thinking, instead of defining what makes an experimental book or what constitutes experimental publishing, we position it here in relation to certain practices and contexts instead. For example, what becomes clear when examining experimental book publishing within academia is that it is (historically) positioned across three interconnected discourses: the codex format, digital publishing, and openness.

As outlined above, experimental forms and practices of publishing open up and explore questions around modalities, linearity, workflow, and the relationalities of publishing; they examine established practices that many scholarly communities have taken for granted and repeated within conventional forms of publishing—where they have become solidified in standard print- and codex-based publishing forms and practices. This especially concerns discussions about what constitutes a publication, or at what point scholarship is formally ‘published’ (the current consensus is that a book is published once it is peer reviewed and published by a reputable press). Unlike Elliott and others, who argue for clear definitions to distinguish experimental publishing, we resist clear definitions, simply because the form and value of experimentation is research-, field- and discourse-specific. Fixed definitions risks closing down conversations that different scholarly communities have to determine their own (what we hope are contingent and continuously reviewed) understandings of what constitutes a book or a publication. In this compendium, we speak of experimental publishing and experimental books to refer to a mode of publishing that might also be called multimodal, screen-based, or interactive. We prefer experimental publishing because it can encompass multi-modal, interactive, and screen-based works, without restricting which media forms or practices count as experimental. Experimental publishing thus invites discussion about the form and intend of publishing experiments. 

We, to collapse our editorial politics for a minute, favour the technofeminist dreamscape of biodiverse swamps and moors where more collective, inclusive, embodied, situated and caring modes of knowledge production may emerge. Pushing for diverse publishing landscapes, we recognise that this dream is limited and particular. The notion that changes in publishing effect the entire scholarly landscape applies equally to pipe dreams of scholarly efficiency and to ways of reimagining scholarly publishing that lies in our blindspot, beyond our horizon. While we know where we stand, we hope that this compendium is useful for all who are working to expand scholarly publishing, working to promote and give visibility to the rich and diverse forms of digital scholarship, and multimodal and interactive research out there and inspire ongoing experimentation. The compendium will be funded for X years by X. During this time at least the compendium will remain under active development. Our hope is that it will help build relationships between software and tool providers, publishers, librarians, researchers, and authors, working with and strengthening communities of expertise around experimental books.

If you are a publisher, developer, initiative, library, institution or author wanting to contribute to the further development of the compendium please get in touch with [compendium@openbookcollective.org](mailto:compendium@openbookcollective.org). 

The compendium grew out of the Books Contain Multitudes: Exploring Experimental Publishing Report. (2022 update). Community-Led Open Publication Infrastructures for Monographs (COPIM). [https://doi.org/10.21428/785a6451.1792b84f](https://doi.org/10.21428/785a6451.1792b84f) & [https://doi.org/10.5281/zenodo.6545475](https://doi.org/10.5281/zenodo.6545475).